<?php
namespace Financeiro\Logic\ImportacaoExtratoBancario;

use \Geral\Logic\OfxParser\OfxParser;
use \Financeiro\Entities\Movreal;
use \Financeiro\Entities\Extrbanco;
use \Geral\Util\Ambiente;
use Geral\Util\DateUtil;
use \Geral\Util\NovoCodigo;
use DateTime;

    /**
 * Created by JetBrains PhpStorm.
 * User: matheussilva
 * Date: 17/12/13
 * Time: 15:31
 * To change this template use File | Settings | File Templates.
 */

  class ExtratoBancario{

      const CONTA = 1;
      const MOVIMENTOS = 2;
      const EXTRATO=3;
      const EXTENSOES_ACEITAS = "ofx";

      /**
       * Amazena o objeto com informações lidas no OFX
       * @var $dados OfxParser
       */
      private $dados;

      /**
       * Executa a função para ler o nome do arquivo.
       * @param $arquivo
       */
      public function __construct($arquivo){
          self::readOfx($arquivo);
      }

      /**
       * Insere zero antes do número até
       * @param $num int
       * @param $qnt int
       * @return string
       */
      public function fillZeroAnt($num, $qnt){
          $num = (string)$num;
          while(strlen($num)<$qnt){
              $num = '0'.$num;
          }
          return $num;
      }

      /**
       * Se o Tamanho da string for menor que 40, retorna a string. Se não, retorna os 40 primeiros caracteres
       * @param $string
       * @return string
       */
      public function verificaTamMemo($string){
          if(strlen($string)>40){
              return (substr($string, 0, 40));
          }else{
              return $string;
          }
      }

      /**
       * Recebe a data e formata para o formato Brasileiro
       * @param $data string
       * @return string
       */
      private function correctDate($data){
          $ano = substr($data, 0, 4);
          $mes = substr($data, 4, 2);
          $dia = substr($data, 6, 2);

          return("$dia/$mes/$ano");
      }

      /**
       * Retorna C para CREDIT e D para DEBIT
       * @param $tipo string
       * @return string
       */
      private function correctType($tipo){
          if($tipo=='CREDIT')
              return 'C';
          else
              return 'D';
      }

      /**
       * Formata os dados do movimento
       * @param $movimentos
       */
      private function setData($movimentos){
          for($c=0;$c<count($movimentos);$c++){
              $movimentos[$c]['DTPOSTED'] = self::correctDate($movimentos[$c]['DTPOSTED']);
              $movimentos[$c]['TRNTYPE'] = self::correctType($movimentos[$c]['TRNTYPE']);
              if($movimentos[$c]["TRNAMT"] > 0){
                  $movimentos[$c]["TRNTYPE"] = "C";
              }
              $movimentos[$c]['TRNAMT'] = abs($movimentos[$c]['TRNAMT']);
          }
          return ($movimentos);
      }

      /**
       * Retorna a soma de todos os movimentos
       * @return int
       */
      private function vrTotalMov(){
          $vrMovs = $this->dados->getMoviments();
          $total = 0;
          foreach($vrMovs as $vrMov){
              $total+= $vrMov['TRNAMT'];
          }
          return $total;
      }

      /**
       * Retorna se o valor é credito ou débito
       * @param $valor float
       * @return string
       */
      private function verificaCredDeb($valor){
          if($valor<0){
              return 'D';
          }else{
              return 'C';
          }
      }

      /**
       * Lê o nome do arquivo, cria e armazema o objeto no atributo $dados
       * @param $caminho string
       */
      private function readOfx($caminho){
          $this->dados = new OfxParser();
          $this->dados->loadFromFile($caminho);
      }

      /**
       * Retorna o conteudo se encontrar, caso não encontre, retorna false
       * @param $tipo
       * @return array|bool
       */
      public function getOfx($tipo){
          switch($tipo){
              case(self::CONTA):{
                  return $this->dados->getBankingData();
              }
              case(self::MOVIMENTOS):{
                  return $movimentos = self::setData($this->dados->getMoviments());
              }
              case(self::EXTRATO):{
                  $extrato = $this->dados->getFiledata();
                  $extrato['DTSTART'] = self::correctDate($extrato['DTSTART']);
                  $extrato['DTEND'] = self::correctDate($extrato['DTEND']);

                  return $extrato;
              }
              default:{
                  return false;
              }
          }
      }

      /**
       * Insere um extrato no banco
       * @throws \Exception
       */
      public function inserirExtrato( ){

          //DEFINIÇÃO DE VARIÁVEIS EXTERNAS

          $nrorg = Ambiente::getNrorg();
          $conta = tk_get_context_vars('REDUCONT', 'win_pbi00169_000');

          //BUSCAR DADOS (CONTEXT DEFINIDO NO FILTRO)

          $sequextr = NovoCodigo::geraCodigo($nrorg, 'MOVREAL' . $conta, 1, 10, false);
          $dtmov = self::getOfx(self::EXTRATO);

          $dtIni = DateUtil::getDataDeString($dtmov['DTSTART']);
          $dtFim = DateUtil::getDataDeString($dtmov['DTEND']);
          $vrSaldFim = $dtmov['BALAMT'];

          $vrSaldIni = $vrSaldFim-self::vrTotalMov();

          //INSERIR NO BANCO
          $extrato = new Extrbanco();

          $extrato->setCdreducont($conta);
          $extrato->setCdsequextr($sequextr);
          $extrato->setDtsaldinic($dtIni);
          $extrato->setDtsaldfin($dtFim);
          $extrato->setVrsaldinic($vrSaldIni);
          $extrato->setVrsaldfim($vrSaldFim);
          $extrato->setIdextrini(self::verificaCredDeb($vrSaldIni));
          $extrato->setIdextrfim(self::verificaCredDeb($vrSaldFim));
          $extrato->setDtinclusao(DateUtil::getDataAtual());
          $extrato->setNrorg($nrorg);

          try {
              $extrato->persist();
              \Setforms2\Shared\Registry::getEntityManager()->flush();
              return $sequextr;
          } catch (\Exception $e) {
              throw $e;
          }

      }

      /**
       * Insere todos os movimentos no banco
       * @param $extrato
       * @throws \Exception
       */
      public function inserirMovimentoReal($extrato){
          try {
              //DEFINIÇÃO DE VARIÁVEIS EXTERNAS (CONTEXT DEFINIDO NO FILTRO)

              $nrorg = Ambiente::getNrorg();
              $reducont = tk_get_context_vars('REDUCONT', 'win_pbi00169_000');

              //BUSCAR DADOS

              $movimentos = self::getOfx(self::MOVIMENTOS);
              $nrseqmovreal = 0;
              $inserir = 0;

              foreach($movimentos as $mov){
                  $nrseqmovreal++;
                  $nrseqmovreal = self::fillZeroAnt($nrseqmovreal, 10);
                  $nrdocmovreal = $mov['CHECKNUM'];
                  $vrmovreal = $mov['TRNAMT'];
                  $idmovreal = $mov['TRNTYPE'];
                  $date = DateUtil::getDataDeString($mov['DTPOSTED']);
                  $date = DateUtil::truncateData($date);

                  /** @var $movreal \Financeiro\Repositories\Movreal */
                  $movreal = Movreal::getRepository();
                  $movimentacao = $movreal->consultaMovimentacao($nrorg, $reducont, $nrdocmovreal, $mov['DTPOSTED'], $vrmovreal, $idmovreal);
                  if(!$movimentacao){
                      //INSERIR NO BANCO
                      $movimento = new Movreal();

                      $movimento->setNrorg($nrorg);
                      $movimento->setCdreducont($reducont);
                      $movimento->setCdsequextr($extrato);
                      $movimento->setNrseqmovreal($nrseqmovreal);
                      $nrdocmovreal = str_pad($nrdocmovreal, 20, 0, STR_PAD_LEFT);
                      $movimento->setNrdocmovreal($nrdocmovreal);
                      /** @var $date DateTime*/
                      $movimento->setDtmovreal($date);
                      $movimento->setVrmovreal($vrmovreal);
                      $movimento->setIdmovreal($idmovreal);
                      $movimento->setDshistmovreal(self::verificaTamMemo($mov['MEMO']));
                      $movimento->setIdtiporegi(0);
                      $movimento->setIdstconmvban(0);
                      $movimento->setIdtpmovreal(0);
                      $movimento->setDtinclusao(DateUtil::getDataAtual());
                      $movimento->setCdoperinclusao(Ambiente::getCdOperador());
                      $movimento->setNrorginclusao($nrorg);

                      $inserir = 1;
                      $movimento->persist();
                      \Setforms2\Shared\Registry::getEntityManager()->flush();
                  }
              }
              return $inserir;
          } catch (\Exception $e) {
              throw $e;
          }
      }

      /**
       * Valida extensão do arquivo para OFX
       * @param $nomeArquivo
       * @return bool
       */
      public static function validaExtensao($nomeArquivo){

          return (bool)preg_match("/\." . self::EXTENSOES_ACEITAS . "/", $nomeArquivo);
      }

      public static function alteraNomeArquivo($nomeArquivo, $caminho){

          try {
              $nrorg = Ambiente::getNrorg();
              $empresa = Ambiente::getCdEmpresa();
              $filial = Ambiente::getCdFilial();
              $user = Ambiente::getCdOperador();
              $data = DateUtil::getDataAtual()->format("HisYmd");

              $nome = $nrorg . $empresa . $filial . $user . $data . $nomeArquivo;
              $nome = md5($nome);

              rename($caminho . $nomeArquivo, $caminho . $nome);
              return ($caminho.$nome);
          } catch (\Exception $e) {
              tk_warning($e->getMessage());
          }
      }

      public static function buscaArquivo(){
          try {
              $caminho = APP_PATH . '\\public\\uploads\\';
              if (self::validaExtensao(tk_get_uploaded_file_name())) {
                  $nomeArquivo = self::alteraNomeArquivo(tk_get_uploaded_file_name(), $caminho);
                  self::alteraNomeArquivo($nomeArquivo, $caminho);
                  return (realpath($nomeArquivo));
              } else {
                  throw new \Exception(tk_get_message_by_name('asfasdfsafsf'));
              }
          } catch (\Exception $e) {
              throw $e;
          }


      }
  }